/*
 * @brief Definitions and declarations of Mass Storage device example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _MASS_STORAGE_H_
#define _MASS_STORAGE_H_
/* Includes: */
		#include "board.h"
		#include "USB.h"
		#include <string.h>
		#include "Descriptors.h"
		#include "Lib/SCSI.h"
		#include "Lib/DataRam.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup Mass_Storage_Device Mass Storage Device
 * @ingroup EXAMPLES_USB_18XX43XX
 * The Mass Storage Device example demonstrates a USB Mass Storage Device
 * using LPCUSBlib library. The MCU will be enumerated as as Mass Storage Device when
 * connected to the Host PC.
 *
 * See readme.txt for building and running guide.
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *  - NGX Xplorer 1830 and 4330 boards
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/** @defgroup Mass_Storage_Device_Definition Main definitions
 * @{
 */
/** LED mask for the library LED driver, to indicate that the USB interface is not ready. */
		#define LEDMASK_USB_NOTREADY      LEDS_LED1

/** LED mask for the library LED driver, to indicate that the USB interface is enumerating. */
		#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is ready. */
		#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

/** LED mask for the library LED driver, to indicate that an error has occurred in the USB interface. */
		#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is busy. */
		#define LEDMASK_USB_BUSY          LEDS_LED2

/** Total number of logical drives within the device - must be non-zero. */
		#define TOTAL_LUNS                1

//		#define VIRTUAL_MEMORY_BYTES                (MassStorage_GetCapacity())
//		#define VIRTUAL_MEMORY_BLOCK_SIZE           512
//		#define VIRTUAL_MEMORY_BLOCKS               (VIRTUAL_MEMORY_BYTES / VIRTUAL_MEMORY_BLOCK_SIZE)
/** Blocks in each LUN, calculated from the total capacity divided by the total number of Logical Units in the device. */
		#define LUN_MEDIA_BLOCKS         (VIRTUAL_MEMORY_BLOCKS / TOTAL_LUNS)

/** Indicates if the disk is write protected or not. */
		#define DISK_READ_ONLY            false
/**
 * @}
 */

/** @defgroup Mass_Storage_Device_Functions Functions prototypes
 * @{
 */
 /**
 * @brief	Hardware setup event callback function
 * @return	None
 * This is the USB HW set up event call back function
 */
void SetupHardware(void);

void EVENT_USB_Device_Connect(void);

void EVENT_USB_Device_Disconnect(void);

void EVENT_USB_Device_ConfigurationChanged(void);

void EVENT_USB_Device_ControlRequest(void);

bool CALLBACK_MS_Device_SCSICommandReceived(USB_ClassInfo_MS_Device_t *const MSInterfaceInfo);

/**
 * @}
 */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif
