/*
 * @brief SDMMC Chan FATFS simple abstraction layer
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */
 
/** Under development, not working yet. */
#ifdef CFG_SDCARD

#include "fs_mci.h"
#include "board.h"
#include "chip.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

/* Disk Status */
static volatile DSTATUS Stat = STA_NOINIT;

/* 100Hz decrement timer stopped at zero (disk_timerproc()) */
static volatile WORD Timer2;

/* Card Configuration */
// CARDCONFIG CardConfig;      /* Card configuration */

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

extern mci_card_struct sdcardinfo;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Get bits from an array */
uint32_t unstuff_bits(uint8_t *resp, uint32_t start, uint32_t size)
{
	uint32_t byte_idx_stx;
	uint8_t bit_idx_stx, bit_idx;
	uint32_t ret, byte_idx;

	byte_idx_stx = start / 8;
	bit_idx_stx = start - byte_idx_stx * 8;

	if (size < (8 - bit_idx_stx)) {
		return (resp[byte_idx_stx] >> bit_idx_stx) & ((1 << size) - 1);
	}

	ret = 0;

	ret =  (resp[byte_idx_stx] >> bit_idx_stx) & ((1 << (8 - bit_idx_stx)) - 1);
	bit_idx = 8 - bit_idx_stx;
	size -= bit_idx;

	byte_idx = 1;
	while (size > 8) {
		ret |= resp[byte_idx_stx + byte_idx] << (bit_idx);
		size -= 8;
		bit_idx += 8;
		byte_idx++;
	}

	if (size > 0) {
		ret |= (resp[byte_idx_stx + byte_idx] & ((1 << size) - 1)) << bit_idx;
	}

	return ret;
}

/* Swap buffer */
static void swap_buff(uint8_t *buff, uint32_t count)
{
	uint8_t tmp;
	uint32_t i;

	for (i = 0; i < count / 2; i++) {
		tmp = buff[i];
		buff[i] = buff[count - i - 1];
		buff[count - i - 1] = tmp;
	}
}

/* Wait for device to complete */
bool mci_wait_for_ready(void)
{
	/* Poll card for program completion */
	Timer2 = 50;// 500 ms
	while (Timer2) {
		if (Chip_SDMMC_GetState() != -1) {
			return true;
		}
	}

	return false;
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Initialize Disk Drive */
DSTATUS disk_initialize(BYTE drv)
{
	if (drv) {
		return STA_NOINIT;				/* Supports only single drive */
	}
	//	if (Stat & STA_NODISK) return Stat;	/* No card in the socket */

	if (Stat != STA_NOINIT) {
		return Stat;					// card is already enumerated

	}
	/* Reset */
	Stat = STA_NOINIT;

#ifndef BOARD_NGX_XPLORER_18304330
	/* NGX board ignored SD_CD pin */
	/* Wait for a card to be inserted (note CD is not on the SDMMC power rail and can be polled without enabling SD slot power */
	while (Chip_SDMMC_CardNDetect() == 0) {}
#endif

	/* Enumerate the card once detected. Note this function may block for a little while. */

	if (!Chip_SDMMC_Acquire(&sdcardinfo)) {
		DEBUGOUT("Card Acquire failed...\r\n");
		return Stat;
	}

	Stat &= ~STA_NOINIT;
	return Stat;

}

/* Disk Drive miscellaneous Functions */
DRESULT disk_ioctl(BYTE drv, BYTE ctrl, void *buff)
{
	DRESULT res;
	BYTE n, *ptr = buff;
	ULONG *ptr32 = (ULONG *) buff;

	if (drv) {
		return RES_PARERR;
	}
	if (Stat & STA_NOINIT) {
		return RES_NOTRDY;
	}

	res = RES_ERROR;

	switch (ctrl) {
	case CTRL_SYNC:	/* Make sure that no pending write process */
		if (mci_wait_for_ready() == true) {
			res = RES_OK;
		}

		break;

	case GET_SECTOR_COUNT:	/* Get number of sectors on the disk (DWORD) */
		*(DWORD *) buff = sdcardinfo.blocknr;
		res = RES_OK;
		break;

	case GET_SECTOR_SIZE:	/* Get R/W sector size (WORD) */
		*(WORD *) buff = sdcardinfo.block_len;	// 512;
		res = RES_OK;
		break;

	case GET_BLOCK_SIZE:/* Get erase block size in unit of sector (DWORD) */
		*(DWORD *) buff = 4096;	// FIXED:
		res = RES_OK;
		break;

	case MMC_GET_TYPE:		/* Get card type flags (1 byte) */
		*ptr = sdcardinfo.card_type;
		res = RES_OK;
		break;

	case MMC_GET_CSD:		/* Receive CSD as a data block (16 bytes) */
		for (n = 0; n < 4; n++) {
			*(ptr32 + n) = sdcardinfo.csd[n];
		}
		res = RES_OK;
		break;

	case MMC_GET_CID:		/* Receive CID as a data block (16 bytes) */
		for (n = 0; n < 4; n++) {
			*(ptr32 + n) = sdcardinfo.cid[n];
		}
		res = RES_OK;
		break;

	case MMC_GET_SDSTAT:/* Receive SD status as a data block (64 bytes) */
	{
		int32_t cardStatus;

		cardStatus = Chip_SDMMC_GetState();
		if (cardStatus != -1) {
			uint8_t *status = (uint8_t *) &cardStatus;
			for (n = 0; n < 2; n++) {
				*(ptr + n) = ((uint8_t *) status)[n];
			}
			res = RES_OK;
		}
	}
	break;

	default:
		res = RES_PARERR;
		break;
	}

	return res;
}

/* Read Sector(s) */
DRESULT disk_read(BYTE drv, BYTE *buff, DWORD sector, BYTE count)
{
	if (drv || !count) {
		return RES_PARERR;
	}
	if (Stat & STA_NOINIT) {
		return RES_NOTRDY;
	}

	if (Chip_SDMMC_ReadBlocks(buff, sector, count)) {
		return RES_OK;
	}

	return RES_ERROR;
}

/* Get Disk Status */
DSTATUS disk_status(BYTE drv)
{
	if (drv) {
		return STA_NOINIT;	/* Supports only single drive */

	}
	return Stat;
}

/* Write Sector(s) */
DRESULT disk_write(BYTE drv, const BYTE *buff, DWORD sector, BYTE count)
{

	if (drv || !count) {
		return RES_PARERR;
	}
	if (Stat & STA_NOINIT) {
		return RES_NOTRDY;
	}

	if ( Chip_SDMMC_WriteBlocks((void *) buff, sector, count)) {
		return RES_OK;
	}

	return RES_ERROR;
}

/* Device timer function */
void disk_timerproc(void)
{
	WORD n;

	n = Timer2;
	if (n) {
		Timer2 = --n;
	}
}
#endif /*CFG_SDCARD*/
