/*
 * @brief SDMMC Chan FATFS simple abstraction layer
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"
#include "diskio.h"

/** @ingroup EXAMPLES_PERIPH_18XX43XX_SDMMC
 * @{
 */

/**
 * @brief	Initialize Disk Drive
 * @param	drv	: Physical drive number
 * @return	Drive status
 */
DSTATUS disk_initialize(BYTE drv);

/**
 * @brief	Disk Drive miscellaneous Functions
 * @param	drv		: Physical drive number
 * @param	ctrl	: Control code
 * @param	buff	: Buffer to send/receive control data
 * @return	Drive status
 */
DRESULT disk_ioctl(BYTE drv, BYTE ctrl, void *buff);

/**
 * @brief	Read Sector(s)
 * @param	drv		: Physical drive number
 * @param	buff	: Pointer to the data buffer to store read data
 * @param	sector	: Start sector number (LBA)
 * @param	count	: Sector count (1..255
 * @return	Drive status
 */
DRESULT disk_read(BYTE drv, BYTE *buff, DWORD sector, BYTE count);

/**
 * @brief	Get Disk Status
 * @param	drv	: Physical drive number
 * @return	Drive status
 */
DSTATUS disk_status(BYTE drv);

/**
 * @brief	Write Sector(s)
 * @param	drv		: Physical drive number
 * @param	buff	: Pointer to the data to be written
 * @param	sector	: Start sector number (LBA)
 * @param	count	: Sector count (1..255
 * @return	Drive status
 */
DRESULT disk_write(BYTE drv, const BYTE *buff, DWORD sector, BYTE count);

/**
 * @brief	Device timer function
 * @return	Nothing
 * This function must be called from timer interrupt routine in period
 * of 10 ms to generate card control timing
 */
void disk_timerproc(void);

/**
 * @}
 */
