/*
 * @brief SD/MMC example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */
 
/** Under development, not working yet. */
#ifdef CFG_SDCARD
#include "sdmmc.h"

/** @defgroup EXAMPLES_PERIPH_18XX43XX_SDMMC SDMMC example
 * @ingroup EXAMPLES_PERIPH_18XX43XX
 * This example describes how to use the SDMMC interface with a SD card
 * and a filesystem.
 *
 * FIXME - need a better description
 *
   Hitex A4 LPC1850-4350:
   - Close pin 1-2 of JP9
   - Open all SV3, SV6
   - Close all SV12
   - Debug output on Uart port 1: SV11 pin 2(U1_RXD) and pin 4(U1_TXD)
   NGX LPC1830-4330
   - Debug output on Uart port 0: J8 pin 9(U0_TXD) and pin 10(U0_RXD)
   Keil MCB1857-4357
   - Debug output on Uart port 0
 *
 *
 * No special setup is required for the demo. Just build and run.
 *
 * Build procedure:
 * <a href="http://www.lpcware.com/content/project/lpcopen-platform-nxp-lpc-microcontrollers/lpcopen-build-procedures/lpc18xx/43xx-lpco">LPCOpen 18xx/43xx build instructions</a>
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *  - NGX Xplorer 1830 and 4330 boards
 * @{
 */

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/

#if _USE_XSTRFUNC == 0
#include <string.h>
#define xstrlen(x)      strlen(x)
#define xstrcpy(x, y)    strcpy(x, y)
#define xmemset(x, y, z)  memset(x, y, z)
#define xstrchr(x, y)    strchr(x, y)
#endif

#ifdef BOARD_HITEX_EVA_18504350
#define debugstr(str)  board_uart_out_string(str)
#else
#define debugstr(str)  DEBUGSTR(str)
#endif

/* buffer size (in byte) for R/W operations */
#define BUFFER_SIZE     4096

static volatile UINT Timer = 0;		/* Performance timer (1kHz increment) */
static volatile int32_t sdio_wait_exit = 0;
static volatile uint32_t u32Milliseconds;
static volatile uint32_t timerCntms = 0;

#define TIME_INTERVAL  1/* 1ms */

/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/* SDMMC card info structure */
mci_card_struct sdcardinfo;

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/* Delay callback for timed SDIF/SDMMC functions */
static void sdmmc_waitms(uint32_t time)
{
	/* In an RTOS, the thread would sleep allowing other threads to run. For standalone operation, we just spin on a timer */
	/*  TIM_Waitus(time * 1000); */
	timerCntms = time;
	while (timerCntms) {}
}

/**
 * @brief	Sets up the SD event driven wakeup
 * @param	bits : Status bits to poll for command completion
 * @return	Nothing
 */
static void sdmmc_setup_wakeup(uint32_t bits)
{
	/* Wait for IRQ - for an RTOS, you would pend on an event here with a IRQ based wakeup. */
	NVIC_ClearPendingIRQ(SDIO_IRQn);
	sdio_wait_exit = 0;
	Chip_SDMMC_SetIntMask(bits);
	NVIC_EnableIRQ(SDIO_IRQn);
}

/**
 * @brief	A better wait callback for SDMMC driven by the IRQ flag
 * @param	bits : Status bits to poll for command completion
 * @return	0 on success, or failure condition (-1)
 */
static uint32_t sdmmc_irq_driven_wait(void)
{
	uint32_t status;

	/* Wait for event, would be nice to have a timeout, but keep it  simple */
	while (sdio_wait_exit == 0) {}

	/* Get status and clear interrupts */
	status = Chip_SDMMC_GetIntStatus();

	return status;
}

/* Initialize the Repetitive Interrupt Timer at 1s */
void App_Timer_Init(void)
{
	Chip_RIT_Init();
	/* Configure time_interval for RIT
	 * In this case: time_interval = 1000 ms = 1s
	 * So, RIT will generate interrupt each 1s
	 */
	Chip_RIT_SetTimerInterval(TIME_INTERVAL);
	/* Initialize RTC timer */
	rtc_initialize();
}

/* Initialize SD/MMC */
void App_SDMMC_Init(void)
{
	memset(&sdcardinfo, 0, sizeof(sdcardinfo));
	sdcardinfo.evsetup_cb = sdmmc_setup_wakeup;
	sdcardinfo.waitfunc_cb = sdmmc_irq_driven_wait;
	sdcardinfo.msdelay_func = sdmmc_waitms;

	/*  SD/MMC initialization */
	Board_SDMMC_Init();

	/* The SDIO driver needs to know the SDIO clock rate */
	Chip_Clock_EnableOpts(CLK_MX_SDIO, true, true, 1);
	sdcardinfo.clk_rate = Chip_Clock_GetRate(CLK_MX_SDIO);
	Chip_SDMMC_Init();
}

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/**
 * @brief	Error processing function: stop with dying message
 * @param	rc	: FatFs return value
 * @return	Nothing
 */
void die(FRESULT rc)
{
	DEBUGOUT("Failed with rc=%u.\n", rc);
	for (;; ) {}
}

/**
 * @brief	System tick interrupt handler
 * @return	Nothing
 */
void SysTick_Handler(void)
{
	u32Milliseconds += 10;
}

/**
 * @brief	SDIO controller interrupt handler
 * @return	Nothing
 */
void SDIO_IRQHandler(void)
{
	/* All SD based register handling is done in the callback
	   function. The SDIO interrupt is not enabled as part of this
	   driver and needs to be enabled/disabled in the callbacks or
	   application as needed. This is to allow flexibility with IRQ
	   handling for applicaitons and RTOSes. */
	/* Set wait exit flag to tell wait function we are ready. In an RTOS,
	   this would trigger wakeup of a thread waiting for the IRQ. */
	NVIC_DisableIRQ(SDIO_IRQn);
	sdio_wait_exit = 1;
}

/**
 * @brief	RIT interrupt handler sub-routine
 * @return	Nothing
 */
void RIT_IRQHandler(void)
{
	/* Clear interrupt flag */
	Chip_RIT_ClearInt();

	/* Disk timer function (100Hz) */
	disk_timerproc();
	if (timerCntms) {
		timerCntms--;
	}

}

/**
 * @brief	User Provided Timer Function for FatFs module
 * @return	Nothing
 * This is a real time clock service to be called from FatFs module.
 * Any valid time must be returned even if the system does not support a real time clock.
 * This is not required in read-only configuration.
 */
DWORD get_fattime()
{
	RTC rtc;

	/* Get local time */
	rtc_gettime(&rtc);

	/* Pack date and time into a DWORD variable */
	return ((DWORD) (rtc.year - 1980) << 25)
		   | ((DWORD) rtc.month << 21)
		   | ((DWORD) rtc.mday << 16)
		   | ((DWORD) rtc.hour << 11)
		   | ((DWORD) rtc.min << 5)
		   | ((DWORD) rtc.sec >> 1);
}
/**
 * @}
 */
#endif /*CFG_SDCARD*/
