/*
 * @brief Definitions and declarations of Virtual Serial device example
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _VIRTUALSERIAL_H_
#define _VIRTUALSERIAL_H_

		#include "board.h"
		#include "USB.h"
		#include <string.h>
		#include <stdio.h>
		#include "Descriptors.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup Virtual_Serial_Device Virtual Serial Device
 * @ingroup EXAMPLES_USB_18XX43XX
 * This example implements a CDC/ACM (USB-to-COM) virtual serial device. The
 * example reads and writes data through a serial port on the chip which is
 * connected to either 9pin UART connector or a dedicated USB-to-COM chip
 * (FTDI) on the board.
 *
 * See readme.txt for building and running guide.
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *  - NGX Xplorer 1830 and 4330 boards
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/** @defgroup Virtual_Serial_Device_Definition Main definitions
 * @{
 */
/** LED mask for the library LED driver, to indicate that the USB interface is not ready. */
		#define LEDMASK_USB_NOTREADY      LEDS_LED1

/** LED mask for the library LED driver, to indicate that the USB interface is enumerating. */
		#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is ready. */
		#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

/** LED mask for the library LED driver, to indicate that an error has occurred in the USB interface. */
		#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

		#define ECHO_CHARACTER_TASK     (0)
		#define CDC_BRIDGE_TASK         (ECHO_CHARACTER_TASK + 1)

/**
 * @}
 */

/** @defgroup Virtual_Serial_Device_Functions Functions prototypes
 * @{
 */
void SetupHardware(void);

void EchoCharater(void);

void CDC_Bridge_Task(void);

void EVENT_USB_Device_Connect(void);

void EVENT_USB_Device_Disconnect(void);

void EVENT_USB_Device_ConfigurationChanged(void);

void EVENT_USB_Device_ControlRequest(void);

/**
 * @}
 */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif
