/*
 * @brief Definitions and declarations of KeyBoard Host
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _KEYBOARD_HOST_H_
#define _KEYBOARD_HOST_H_

		#include "board.h"
		#include "USB.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup Virtual_Serial_Device_and_Keyboard_Host Virtual Serial Device and Keyboard Host
 * @ingroup EXAMPLES_USB_18XX43XX
 * This example implements a Dual Mode USB device. 
 * check the @ref Virtual_Serial_Device for Device mode.
 * check the @ref Keyboard_Host for Host mode.
 *
 * @note By default, device is in Device mode and there is no automatic switching between USB modes.
 *		-	To switch from Device to Host mode, send a character 'h' on Virtual Serial Port.
 *		-	To switch back from Host to Device mode, press 'D' on Keyboard when it is attached and
 *			enumerated on Host USB port.
 * @note Since Device mode isn't support Dual port, this example in Device mode can only work on the port 
 *		that specified by @ref USB_PORT.
 *
 * See readme.txt for building and running guide.
 *
 * Supported boards and board setup:
 *  - Hitex LPC1850EVA-A4-2 and LPC4350EVA-A4-2 boards
 *  - Keil MCB1857 and MCB4357 boards
 *  - NGX Xplorer 1830 and 4330 boards
 *
 * Submit bug reports for LPCOpen code <a href="http://www.lpcware.com/content/bugtrackerproject/lpcopen">here.</a>
 * @{
 */

/** @defgroup Keyboard_Host_Definition Main definitions for Keyboard Host
 * @{
 */
/** LED mask for the library LED driver, to indicate that the USB interface is not ready. */
		#define LEDMASK_USB_NOTREADY      LEDS_LED1

/** LED mask for the library LED driver, to indicate that the USB interface is enumerating. */
		#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is ready. */
		#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

/** LED mask for the library LED driver, to indicate that an error has occurred in the USB interface. */
		#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

/**
 * @}
 */

/** @defgroup Keyboard_Host_Functions Functions prototypes
 * @{
 */

char keyboard(void);

void SetupHardwareKeyboard(void);

char KeyboardHost_Task(void);

void EVENT_USB_Host_HostError(const uint8_t corenum, const uint8_t ErrorCode);

void EVENT_USB_Host_DeviceAttached(const uint8_t corenum);

void EVENT_USB_Host_DeviceUnattached(const uint8_t corenum);

void EVENT_USB_Host_DeviceEnumerationFailed(const uint8_t corenum,
											const uint8_t ErrorCode,
											const uint8_t SubErrorCode);

void EVENT_USB_Host_DeviceEnumerationComplete(const uint8_t corenum);

/**
 * @}
 */

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif
