/*
 * @brief Definitions and declarations of Virtual Serial Device
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _VIRTUALSERIAL_H_
#define _VIRTUALSERIAL_H_

		#include "board.h"
		#include "USB.h"
		#include <string.h>
		#include <stdio.h>
		#include "Descriptors.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @ingroup Virtual_Serial_Device_and_Keyboard_Host
 */

/** @defgroup Virtual_Serial_Device_Definition Main definitions for Virtual Serial
 * @{
 */
/** LED mask for the library LED driver, to indicate that the USB interface is not ready. */
		#define LEDMASK_USB_NOTREADY      LEDS_LED1

/** LED mask for the library LED driver, to indicate that the USB interface is enumerating. */
		#define LEDMASK_USB_ENUMERATING  (LEDS_LED2 | LEDS_LED3)

/** LED mask for the library LED driver, to indicate that the USB interface is ready. */
		#define LEDMASK_USB_READY        (LEDS_LED2 | LEDS_LED4)

/** LED mask for the library LED driver, to indicate that an error has occurred in the USB interface. */
		#define LEDMASK_USB_ERROR        (LEDS_LED1 | LEDS_LED3)

		#define ECHO_CHARACTER_TASK     (0)
		#define CDC_BRIDGE_TASK         (ECHO_CHARACTER_TASK + 1)

/**
 * @}
 */

/** @defgroup Virtual_Serial_Device_Functions Functions prototypes
 * @{
 */
void SetupHardware(void);

char EchoCharater(void);

void CDC_Bridge_Task(void);

void EVENT_USB_Device_Connect(void);

void EVENT_USB_Device_Disconnect(void);

void EVENT_USB_Device_ConfigurationChanged(void);

void EVENT_USB_Device_ControlRequest(void);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif
