/*
 * @brief Architectures' specific definitions
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * Copyright(C) Dean Camera, 2011, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

/** @ingroup Group_Common
 *  @defgroup Group_ArchitectureSpecific Architecture Specific Definitions
 *  @brief Architecture specific definitions relating to specific processor architectures.
 *
 *  Architecture specific macros, functions and other definitions, which relate to specific architectures. This
 *  definitions may or may not be available in some form on other architectures, and thus should be protected by
 *  preprocessor checks in portable code to prevent compile errors.
 *
 *  @{
 */

#ifndef __LPCUSBlib_ARCHSPEC_H__
#define __LPCUSBlib_ARCHSPEC_H__

	/* Preprocessor Checks: */
		#if !defined(__INCLUDE_FROM_COMMON_H)
			#error Do not include this file directly. Include LPCUSBlib/Common/Common.h instead to gain this functionality.
		#endif

	/* Enable C linkage for C++ Compilers: */
		#if defined(__cplusplus)
			extern "C" {
		#endif

	/* Public Interface - May be used in end-application: */
		/* Macros: */
			#if (ARCH == ARCH_AVR8) || defined(__DOXYGEN__)

				#define JTAG_DISABLE()                  MACROS{                                      \
				                                                __asm__ __volatile__ (               \
				                                                "in __tmp_reg__,__SREG__" "\n\t"     \
				                                                "cli" "\n\t"                         \
				                                                "out %1, %0" "\n\t"                  \
				                                                "out __SREG__, __tmp_reg__" "\n\t"   \
				                                                "out %1, %0" "\n\t"                  \
				                                                :                                    \
				                                                : "r" (1 << JTD),                    \
				                                                  "M" (_SFR_IO_ADDR(MCUCR))          \
				                                                : "r0");                             \
				                                        }MACROE
			
				#define JTAG_DEBUG_POINT()              __asm__ __volatile__ ("nop" ::)

				#define JTAG_DEBUG_BREAK()              __asm__ __volatile__ ("break" ::)

				#define JTAG_ASSERT(Condition)          MACROS{ if (!(Condition)) { JTAG_DEBUG_BREAK(); } }MACROE

				#define STDOUT_ASSERT(Condition)        MACROS{ if (!(x)) { printf_P(PSTR("%s: Function \"%s\", Line %d: "   \
				                                                "Assertion \"%s\" failed.\r\n"),     \
				                                                __FILE__, __func__, __LINE__, #Condition); } }MACROE

				#if !defined(pgm_read_ptr) || defined(__DOXYGEN__)
					#define pgm_read_ptr(Address)        (void*)pgm_read_word(Address)
				#endif
			#endif
			
	/* Disable C linkage for C++ Compilers: */
		#if defined(__cplusplus)
			}
		#endif

#endif

/** @} */

