/*
 * @brief USB Endpoint definitions for the LPC17xx microcontrollers
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

/** @ingroup Group_EndpointRW
 *  @defgroup Group_EndpointRW_LPC17xx Endpoint Data Reading and Writing (LPC17xx)
 *  @brief Endpoint data read/write definitions for the LPC architecture.
 *
 *  Functions, macros, variables, enums and types related to data reading and writing from and to endpoints.
 */

/** @ingroup Group_EndpointPrimitiveRW
 *  @defgroup Group_EndpointPrimitiveRW_LPC17xx Read/Write of Primitive Data Types (LPC17xx)
 *  @brief Endpoint primitive read/write definitions for the LPC17xx architecture.
 *
 *  Functions, macros, variables, enums and types related to data reading and writing of primitive data types
 *  from and to endpoints.
 */

/** @ingroup Group_EndpointPacketManagement
 *  @defgroup Group_EndpointPacketManagement_LPC17xx Endpoint Packet Management (LPC17xx)
 *  @brief Endpoint packet management definitions for the NXP LPC17xx architecture.
 *
 *  Functions, macros, variables, enums and types related to packet management of endpoints.
 */

/** @ingroup Group_EndpointManagement
 *  @defgroup Group_EndpointManagement_LPC17xx Endpoint Management (LPC17xx)
 *  @brief Endpoint management definitions for the LPC17xx architecture.
 *
 *  Functions, macros and enums related to endpoint management when in USB Device mode. This
 *  module contains the endpoint management macros, as well as endpoint interrupt and data
 *  send/receive functions for various data types.
 *
 *  @{
 */

#ifndef __ENDPOINT_LPC17XX_H__
#define __ENDPOINT_LPC17XX_H__

		#include "../EndpointCommon.h"

		#if defined(__cplusplus)
extern "C" {
		#endif

		#if !defined(__INCLUDE_FROM_USB_DRIVER)
			#error Do not include this file directly. Include lpcroot/libraries/LPCUSBlib/Drivers/USB/USB.h instead.
		#endif

	#if !defined(__DOXYGEN__)

			#define ENDPOINT_DETAILS_MAXEP      6							/* Maximum of supported endpoint */
			#define USED_PHYSICAL_ENDPOINTS     (ENDPOINT_DETAILS_MAXEP * 2)	/* This macro effect memory size of the DCD */

extern volatile bool SETUPReceived;
extern DMADescriptor dmaDescriptor[USED_PHYSICAL_ENDPOINTS];

extern void SIE_WriteCommandData (uint32_t cmd, uint32_t val);

extern void SIE_WriteCommamd (uint32_t cmd);

extern volatile bool isOutReceived;
extern volatile bool isInReady;

void WriteControlEndpoint(uint8_t *pData, uint32_t cnt);

void ReadControlEndpoint(uint8_t *pData);

void DcdDataTransfer(uint8_t PhyEP, uint8_t *pData, uint32_t cnt);

void Endpoint_Streaming(uint8_t *buffer, uint16_t packetsize,
						uint16_t totalpackets, uint16_t dummypackets);

void Endpoint_ClearEndpoints(void);

bool Endpoint_ConfigureEndpoint_Prv(const uint8_t Number,
									const uint8_t UECFG0XData,
									const uint8_t UECFG1XData);

	#endif

/*static inline */ bool Endpoint_ConfigureEndpoint(const uint8_t Number,
												   const uint8_t Type,
												   const uint8_t Direction,
												   const uint16_t Size,
												   const uint8_t Banks)	/*ATTR_ALWAYS_INLINE*/;

static inline void Endpoint_ResetEndpoint(const uint8_t EndpointNumber) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ResetEndpoint(const uint8_t EndpointNumber)
{}

static inline void Endpoint_EnableEndpoint(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_EnableEndpoint(void)
{}

static inline void Endpoint_DisableEndpoint(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_DisableEndpoint(void)
{}

static inline bool Endpoint_IsEnabled(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsEnabled(void)
{
	return true;
}

static inline uint8_t Endpoint_GetBusyBanks(void) ATTR_ALWAYS_INLINE ATTR_WARN_UNUSED_RESULT;

static inline uint8_t Endpoint_GetBusyBanks(void)
{
	return 0;
}

static inline void Endpoint_AbortPendingIN(void)
{}

static inline bool Endpoint_IsConfigured(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsConfigured(void)
{
	return true;
}

static inline uint8_t Endpoint_GetEndpointInterrupts(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline uint8_t Endpoint_GetEndpointInterrupts(void)
{
	return 0;
}

static inline bool Endpoint_HasEndpointInterrupted(const uint8_t EndpointNumber) ATTR_WARN_UNUSED_RESULT
ATTR_ALWAYS_INLINE;

static inline bool Endpoint_HasEndpointInterrupted(const uint8_t EndpointNumber)
{
	return (Endpoint_GetEndpointInterrupts() & (1 << EndpointNumber)) ? true : false;
}

static inline uint16_t Endpoint_BytesInEndpoint(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline uint16_t Endpoint_BytesInEndpoint(void)
{
	if (endpointselected == ENDPOINT_CONTROLEP) {
		return usb_data_buffer_size;
	}
	else {
		// return (dmaDescriptor[ endpointhandle[endpointselected] ].PresentCount);
		return usb_data_buffer_OUT_size;
	}
}

static inline bool Endpoint_IsINReady(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsINReady(void)
{
	if (endpointselected == ENDPOINT_CONTROLEP) {

		return isInReady;
	}
	else {
		uint8_t SelEP_Data;
		if (dmaDescriptor[endpointhandle[endpointselected]].Retired == true) {
			SIE_WriteCommamd(CMD_SEL_EP(endpointhandle[endpointselected]) );
			SelEP_Data = SIE_ReadCommandData(DAT_SEL_EP(endpointhandle[endpointselected]) );
			if ((SelEP_Data & 1) == 0) {
				return true;
			}
		}
		return false;
	}

}

static inline bool Endpoint_IsOUTReceived(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsOUTReceived(void)
{
	if (endpointselected == ENDPOINT_CONTROLEP) {

		return isOutReceived;
	}
	else {
		return (dmaDescriptor[endpointhandle[endpointselected]].Retired &&
				(dmaDescriptor[endpointhandle[endpointselected]].Status == 2 ||
				 dmaDescriptor[endpointhandle[endpointselected]].Status == 3)
				) ? true : false;
	}
}

static inline bool Endpoint_IsSETUPReceived(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsSETUPReceived(void)
{
	return SETUPReceived;
}

static inline void Endpoint_ClearSETUP(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ClearSETUP(void)
{
	SETUPReceived = FALSE;
	usb_data_buffer_index = 0;
	usb_data_buffer_size = 0;
	SIE_WriteCommamd(CMD_SEL_EP(ENDPOINT_CONTROLEP));
	SIE_WriteCommamd(CMD_CLR_BUF);
}

static inline void Endpoint_ClearIN(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ClearIN(void)
{
	uint8_t PhyEP = (endpointselected == ENDPOINT_CONTROLEP ? 1 : endpointhandle[endpointselected]);

	if (endpointselected == ENDPOINT_CONTROLEP) {
		WriteControlEndpoint(usb_data_buffer, usb_data_buffer_index);
		usb_data_buffer_index = 0;
		usb_data_buffer_size = 0;
	}
	else {
		DcdDataTransfer(PhyEP, usb_data_buffer_IN, usb_data_buffer_IN_index);
		LPC_USB->USBDMARSet = _BIT(PhyEP);
		usb_data_buffer_IN_index = 0;
	}
}

static inline void Endpoint_ClearOUT(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ClearOUT(void)
{
	usb_data_buffer_index = 0;
	if (endpointselected == ENDPOINT_CONTROLEP) {				/* Control only */
		SIE_WriteCommamd(CMD_SEL_EP(ENDPOINT_CONTROLEP));
		SIE_WriteCommamd(CMD_CLR_BUF);
		isOutReceived = false;
	}
	else {
		usb_data_buffer_OUT_index = 0;
		usb_data_buffer_OUT_size = 0;
		dmaDescriptor[endpointhandle[endpointselected]].Status = 0;
		LPC_USB->USBDMAIntEn |= (1 << 1);
	}
}

// static inline void Endpoint_StallTransaction(void) ATTR_ALWAYS_INLINE;
void Endpoint_StallTransaction(void);

static inline void Endpoint_ClearStall(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ClearStall(void)
{
	uint8_t PhysicalEp = endpointhandle[endpointselected] + (endpointselected == ENDPOINT_CONTROLEP ? 1 : 0);

	HAL_DisableUSBInterrupt(USBPortNum);
	SIE_WriteCommandData(CMD_SET_EP_STAT(PhysicalEp), DAT_WR_BYTE(0));
	HAL_EnableUSBInterrupt(USBPortNum);
}

static inline bool Endpoint_IsStalled(void) ATTR_WARN_UNUSED_RESULT ATTR_ALWAYS_INLINE;

static inline bool Endpoint_IsStalled(void)
{
	bool isStalled;

	HAL_DisableUSBInterrupt(USBPortNum);
	SIE_WriteCommamd(CMD_SEL_EP(endpointhandle[endpointselected]) );
	isStalled = SIE_ReadCommandData(DAT_SEL_EP(endpointhandle[endpointselected]) ) & EP_SEL_ST ? true : false;
	HAL_EnableUSBInterrupt(USBPortNum);

	return isStalled;					/* Device Status */
}

static inline void Endpoint_ResetDataToggle(void) ATTR_ALWAYS_INLINE;

static inline void Endpoint_ResetDataToggle(void)
{}

			#if (!defined(FIXED_CONTROL_ENDPOINT_SIZE) || defined(__DOXYGEN__))
extern uint8_t USB_Device_ControlEndpointSize;
			#else
				#define USB_Device_ControlEndpointSize FIXED_CONTROL_ENDPOINT_SIZE
			#endif

void Endpoint_ClearStatusStage(void);

uint8_t Endpoint_WaitUntilReady(void);

		#if defined(__cplusplus)
}
		#endif

#endif

/** @} */

