/*
 * @brief USB Endpoint definitions for the LPC18xx microcontrollers
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#define  __INCLUDE_FROM_USB_DRIVER
#include "../../../USBMode.h"

#if (defined(__LPC18XX__) || defined(__LPC43XX__)) && defined(USB_CAN_BE_DEVICE)
#include "../../../Endpoint.h"
#include <string.h>

#if defined(USB_DEVICE_ROM_DRIVER)
PRAGMA_ALIGN_2048
uint8_t usb_RomDriver_buffer[ROMDRIVER_MEM_SIZE] ATTR_ALIGNED(2048) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_4
uint8_t usb_RomDriver_MSC_buffer[ROMDRIVER_MSC_MEM_SIZE] ATTR_ALIGNED(4) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_4
uint8_t usb_RomDriver_CDC_buffer[ROMDRIVER_CDC_MEM_SIZE] ATTR_ALIGNED(4) __DATA(USBRAM_SECTION);
/** Endpoint IN buffer, used for DMA operation */
PRAGMA_ALIGN_4
uint8_t UsbdCdc_EPIN_buffer[CDC_MAX_BULK_EP_SIZE] ATTR_ALIGNED(4) __DATA(USBRAM_SECTION);
/** Endpoint OUT buffer, used for DMA operation */
PRAGMA_ALIGN_4
uint8_t UsbdCdc_EPOUT_buffer[CDC_MAX_BULK_EP_SIZE] ATTR_ALIGNED(4) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_4
uint8_t usb_RomDriver_HID_buffer[ROMDRIVER_HID_MEM_SIZE] ATTR_ALIGNED(4) __DATA(USBRAM_SECTION);

#endif

#define STREAM_TDs      16

PRAGMA_ALIGN_2048
volatile DeviceQueueHead dQueueHead[USED_PHYSICAL_ENDPOINTS] ATTR_ALIGNED(2048) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_32
DeviceTransferDescriptor dTransferDescriptor[USED_PHYSICAL_ENDPOINTS] ATTR_ALIGNED(32) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_32
DeviceTransferDescriptor dStreamTD[STREAM_TDs] ATTR_ALIGNED(32) __DATA(USBRAM_SECTION);
PRAGMA_ALIGN_4
uint8_t iso_buffer[512] ATTR_ALIGNED(4);

PRAGMA_WEAK(CALLBACK_HAL_GetISOBufferAddress, Dummy_EPGetISOAddress)
uint32_t CALLBACK_HAL_GetISOBufferAddress(const uint32_t EPNum, uint32_t *last_packet_size) ATTR_WEAK ATTR_ALIAS(
	Dummy_EPGetISOAddress);

/* Device transfer completed event
 * Event is required for using the device stack with any RTOS
 */
PRAGMA_WEAK(EVENT_USB_Device_TransferComplete,Dummy_EVENT_USB_Device_TransferComplete)
void EVENT_USB_Device_TransferComplete(int logicalEP, int xfer_in) ATTR_WEAK ATTR_ALIAS(Dummy_EVENT_USB_Device_TransferComplete);

void DcdInsertTD(uint32_t head, uint32_t newtd);

void DcdPrepareTD(DeviceTransferDescriptor *pDTD, uint8_t *pData, uint32_t length, uint8_t IOC);

uint32_t stream_buffer_address, stream_dummy_buffer_address, stream_remain_packets,
		 stream_dummy_packets, stream_packet_size, stream_total_packets;

void HAL_Reset(void)
{
	uint32_t i;

	/* disable all EPs */
	USB_REG(USBPortNum)->ENDPTCTRL[0] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
	USB_REG(USBPortNum)->ENDPTCTRL[1] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
	USB_REG(USBPortNum)->ENDPTCTRL[2] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
	USB_REG(USBPortNum)->ENDPTCTRL[3] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
	if (USB_REG(USBPortNum) == LPC_USB0) {
		USB_REG(USBPortNum)->ENDPTCTRL[4] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
		USB_REG(USBPortNum)->ENDPTCTRL[5] &= ~(ENDPTCTRL_RxEnable | ENDPTCTRL_TxEnable);
	}

	/* Clear all pending interrupts */
	USB_REG(USBPortNum)->ENDPTNAK     = 0xFFFFFFFF;
	USB_REG(USBPortNum)->ENDPTNAKEN       = 0;
	USB_REG(USBPortNum)->USBSTS_D     = 0xFFFFFFFF;
	USB_REG(USBPortNum)->ENDPTSETUPSTAT   = USB_REG(USBPortNum)->ENDPTSETUPSTAT;
	USB_REG(USBPortNum)->ENDPTCOMPLETE    = USB_REG(USBPortNum)->ENDPTCOMPLETE;
	while (USB_REG(USBPortNum)->ENDPTPRIME) ;				/* Wait until all bits are 0 */
	USB_REG(USBPortNum)->ENDPTFLUSH = 0xFFFFFFFF;
	while (USB_REG(USBPortNum)->ENDPTFLUSH) ;	/* Wait until all bits are 0 */

	/* Set the interrupt Threshold control interval to 0 */
	USB_REG(USBPortNum)->USBCMD_D &= ~0x00FF0000;

	/* Configure the Endpoint List Address */
	/* make sure it in on 64 byte boundary !!! */
	/* init list address */
	USB_REG(USBPortNum)->ENDPOINTLISTADDR = (uint32_t) dQueueHead;

	/* Enable interrupts: USB interrupt, error, port change, reset, suspend, NAK interrupt */
	USB_REG(USBPortNum)->USBINTR_D =  USBINTR_D_UsbIntEnable | USBINTR_D_UsbErrorIntEnable |
									 USBINTR_D_PortChangeIntEnable | USBINTR_D_UsbResetEnable |
									 USBINTR_D_SuspendEnable | USBINTR_D_NAKEnable | USBINTR_D_SofReceivedEnable;

	USB_Device_SetDeviceAddress(0);

	endpointselected = 0;
	for (i = 0; i < ENDPOINT_TOTAL_ENDPOINTS; i++)
		endpointhandle[i] = 0;

	usb_data_buffer_size = 0;
	usb_data_buffer_index = 0;

	usb_data_buffer_OUT_size = 0;
	usb_data_buffer_OUT_index = 0;

	// usb_data_buffer_IN_size = 0;
	usb_data_buffer_IN_index = 0;
	stream_total_packets = 0;
}

bool Endpoint_ConfigureEndpoint(const uint8_t Number, const uint8_t Type,
								const uint8_t Direction, const uint16_t Size, const uint8_t Banks)
{
	uint8_t *ISO_Address;
	uint32_t PhyEP = 2 * Number + (Direction == ENDPOINT_DIR_OUT ? 0 : 1);
	uint32_t EndPtCtrl = ENDPTCTRL_REG(Number);

	memset((void *) &dQueueHead[PhyEP], 0, sizeof(DeviceQueueHead) );

	dQueueHead[PhyEP].MaxPacketSize = Size & 0x3ff;
	dQueueHead[PhyEP].IntOnSetup = 1;
	dQueueHead[PhyEP].ZeroLengthTermination = 1;
	dQueueHead[PhyEP].overlay.NextTD = LINK_TERMINATE;

	if (Direction == ENDPOINT_DIR_OUT) {
		EndPtCtrl &= ~0x0000FFFF;
		EndPtCtrl |= ((Type << 2) & ENDPTCTRL_RxType) | ENDPTCTRL_RxEnable | ENDPTCTRL_RxToggleReset;
		if (Type == EP_TYPE_ISOCHRONOUS) {
			uint32_t size = 0;
			ENDPTCTRL_REG(Number) = (Type << 2);					// TODO dummy to let DcdDataTransfer() knows iso transfer
			ISO_Address = (uint8_t *) CALLBACK_HAL_GetISOBufferAddress(Number, &size);
			DcdDataTransfer(PhyEP, ISO_Address, USB_DATA_BUFFER_TEM_LENGTH);
		}
		else {
			USB_REG(USBPortNum)->ENDPTNAKEN |=  (1 << EP_Physical2BitPosition(PhyEP));
		}
	}
	else {	/* ENDPOINT_DIR_IN */
		EndPtCtrl &= ~0xFFFF0000;
		EndPtCtrl |= ((Type << 18) & ENDPTCTRL_TxType) | ENDPTCTRL_TxEnable | ENDPTCTRL_TxToggleReset;
		if (Type == EP_TYPE_ISOCHRONOUS) {
			uint32_t size = 0;
			ENDPTCTRL_REG(Number) = (Type << 18);					// TODO dummy to let DcdDataTransfer() knows iso transfer
			ISO_Address = (uint8_t *) CALLBACK_HAL_GetISOBufferAddress(Number, &size);
			DcdDataTransfer(PhyEP, ISO_Address, size);
		}
	}
	ENDPTCTRL_REG(Number) = EndPtCtrl;

	endpointhandle[Number] = (Number == ENDPOINT_CONTROLEP) ? ENDPOINT_CONTROLEP : PhyEP;
	return true;
}

void Endpoint_Streaming(uint8_t *buffer, uint16_t packetsize,
						uint16_t totalpackets, uint16_t dummypackets)
{
	uint8_t PhyEP = endpointhandle[endpointselected];
	uint16_t i;
#if 0
	for (i = 0; i < totalpackets; i++) {
		DcdDataTransfer(PhyEP, (uint8_t *) (buffer + i * packetsize), packetsize);
		while (!(
				   (dQueueHead[PhyEP].overlay.NextTD & LINK_TERMINATE)
				   && (dQueueHead[PhyEP].overlay.Active == 0)
				   )
			   ) ;
	}
	for (i = 0; i < dummypackets; i++) {
		DcdDataTransfer(PhyEP, buffer, packetsize);
		while (!(
				   (dQueueHead[PhyEP].overlay.NextTD & LINK_TERMINATE)
				   && (dQueueHead[PhyEP].overlay.Active == 0)
				   )
			   ) ;
	}
#else
	uint16_t cnt = 0;
	dummypackets = dummypackets;
	while ( USB_REG(USBPortNum)->ENDPTSTAT & _BIT(EP_Physical2BitPosition(PhyEP) ) ) {	/* Endpoint is already primed */
	}

	for (i = 0; i < totalpackets; i++) {
		uint8_t ioc;
		if (i == STREAM_TDs) {
			break;
		}
		if ((i == totalpackets - 1) || (i == STREAM_TDs - 1)) {
			ioc = 1;
		}
		else {
			ioc = 0;
		}

		DcdPrepareTD(&dStreamTD[i], (uint8_t *) (buffer + i * packetsize), packetsize, ioc);
		if (i > 0) {
			DcdInsertTD((uint32_t) dStreamTD, (uint32_t) &dStreamTD[i]);
		}
		cnt++;
	}

	if (STREAM_TDs < totalpackets) {
		stream_remain_packets = totalpackets - STREAM_TDs;
		stream_buffer_address = (uint32_t) buffer + STREAM_TDs * packetsize;
		stream_packet_size = packetsize;
	}
	else {
		stream_remain_packets = stream_buffer_address = stream_packet_size = 0;
	}
	stream_total_packets = totalpackets;

	dQueueHead[PhyEP].overlay.Halted = 0;	/* this should be in USBInt */
	dQueueHead[PhyEP].overlay.Active = 0;	/* this should be in USBInt */
	dQueueHead[PhyEP].overlay.NextTD = (uint32_t) dStreamTD;
	dQueueHead[PhyEP].TransferCount = totalpackets * packetsize;
	dQueueHead[PhyEP].IsOutReceived = 0;

	USB_REG(USBPortNum)->ENDPTPRIME |= _BIT(EP_Physical2BitPosition(PhyEP) );
#endif
}

void DcdInsertTD(uint32_t head, uint32_t newtd)
{
	DeviceTransferDescriptor *pTD = (DeviceTransferDescriptor *) head;
	while (!(pTD->NextTD & LINK_TERMINATE)) pTD = (DeviceTransferDescriptor *) pTD->NextTD;
	pTD->NextTD = newtd;
}

void DcdPrepareTD(DeviceTransferDescriptor *pDTD, uint8_t *pData, uint32_t length, uint8_t IOC)
{
	/* Zero out the device transfer descriptors */
	memset((void *) pDTD, 0, sizeof(DeviceTransferDescriptor));

	pDTD->NextTD = LINK_TERMINATE;
	pDTD->TotalBytes = length;
	pDTD->IntOnComplete = IOC;
	pDTD->Active = 1;
	pDTD->BufferPage[0] = (uint32_t) pData;
	pDTD->BufferPage[1] = ((uint32_t) pData + 0x1000) & 0xfffff000;
	//	pDTD->BufferPage[2] = ((uint32_t) pData + 0x2000) & 0xfffff000;
	//	pDTD->BufferPage[3] = ((uint32_t) pData + 0x3000) & 0xfffff000;
	//	pDTD->BufferPage[4] = ((uint32_t) pData + 0x4000) & 0xfffff000;
}

void DcdDataTransfer(uint8_t PhyEP, uint8_t *pData, uint32_t length)
{
	DeviceTransferDescriptor *pDTD = (DeviceTransferDescriptor *) &dTransferDescriptor[PhyEP];

	while ( USB_REG(USBPortNum)->ENDPTSTAT & _BIT(EP_Physical2BitPosition(PhyEP) ) ) {	/* Endpoint is already primed */
	}

	/* Zero out the device transfer descriptors */
	memset((void *) pDTD, 0, sizeof(DeviceTransferDescriptor));

	if (((ENDPTCTRL_REG(PhyEP / 2) >> 2) & EP_TYPE_MASK) == EP_TYPE_ISOCHRONOUS) {	// iso out endpoint
		uint32_t mult = (USB_DATA_BUFFER_TEM_LENGTH + 1024) / 1024;
		pDTD->NextTD = LINK_TERMINATE;
		dQueueHead[PhyEP].Mult = mult;
	}
	else if (((ENDPTCTRL_REG(PhyEP / 2) >> 18) & EP_TYPE_MASK) == EP_TYPE_ISOCHRONOUS) {// iso in endpoint
		uint32_t mult = (USB_DATA_BUFFER_TEM_LENGTH + 1024) / 1024;
		pDTD->NextTD = LINK_TERMINATE;
		dQueueHead[PhyEP].Mult = mult;
	}
	else {																		// other endpoint types
		pDTD->NextTD = LINK_TERMINATE;	/* The next DTD pointer is INVALID */
	}
	pDTD->TotalBytes = length;
	pDTD->IntOnComplete = 1;
	pDTD->Active = 1;

	pDTD->BufferPage[0] = (uint32_t) pData;
	pDTD->BufferPage[1] = ((uint32_t) pData + 0x1000) & 0xfffff000;
	pDTD->BufferPage[2] = ((uint32_t) pData + 0x2000) & 0xfffff000;
	pDTD->BufferPage[3] = ((uint32_t) pData + 0x3000) & 0xfffff000;
	pDTD->BufferPage[4] = ((uint32_t) pData + 0x4000) & 0xfffff000;

	dQueueHead[PhyEP].overlay.Halted = 0;	/* this should be in USBInt */
	dQueueHead[PhyEP].overlay.Active = 0;	/* this should be in USBInt */
	dQueueHead[PhyEP].overlay.NextTD = (uint32_t) &dTransferDescriptor[PhyEP];
	dQueueHead[PhyEP].TransferCount = length;

	/* prime the endpoint for transmit */
	USB_REG(USBPortNum)->ENDPTPRIME |= _BIT(EP_Physical2BitPosition(PhyEP) );
}

void TransferCompleteISR(void)
{
	uint8_t *ISO_Address;
	uint32_t ENDPTCOMPLETE = USB_REG(USBPortNum)->ENDPTCOMPLETE;
	USB_REG(USBPortNum)->ENDPTCOMPLETE = ENDPTCOMPLETE;

	if (ENDPTCOMPLETE) {
		uint8_t n;
		for (n = 0; n < USED_PHYSICAL_ENDPOINTS / 2; n++) {	/* LOGICAL */
			if ( ENDPTCOMPLETE & _BIT(n) ) {/* OUT */
				if (((ENDPTCTRL_REG(n) >> 2) & EP_TYPE_MASK) == EP_TYPE_ISOCHRONOUS) {	// iso out endpoint
					uint32_t size = dQueueHead[2 * n].TransferCount;
                                        size -= dQueueHead[2 * n].overlay.TotalBytes;
					// copy to share buffer
					ISO_Address = (uint8_t *) CALLBACK_HAL_GetISOBufferAddress(n, &size);
					DcdDataTransfer(2 * n, ISO_Address, USB_DATA_BUFFER_TEM_LENGTH);
				}
				else {
                                        uint32_t tem = dQueueHead[2 * n].overlay.TotalBytes;
					dQueueHead[2 * n].TransferCount -= tem;

					if (stream_total_packets > 0) {
						if (stream_remain_packets > 0) {
							uint32_t cnt = dQueueHead[2 * n].TransferCount;
							Endpoint_Streaming((uint8_t *) stream_buffer_address,
											   stream_packet_size,
											   stream_remain_packets,
											   0);
							dQueueHead[2 * n].TransferCount = cnt;
						}
						else {
							stream_total_packets = 0;
						}
					}
					else {
						stream_total_packets = 0;
						dQueueHead[2 * n].IsOutReceived = 1;
					}
					if (n == 0) {
						usb_data_buffer_size = dQueueHead[2 * n].TransferCount;
					}
					else {
						usb_data_buffer_OUT_size = dQueueHead[2 * n].TransferCount;
					}
				}
				EVENT_USB_Device_TransferComplete(n, 0);
			}
			if ( ENDPTCOMPLETE & _BIT( (n + 16) ) ) {	/* IN */
				if (((ENDPTCTRL_REG(n) >> 18) & EP_TYPE_MASK) == EP_TYPE_ISOCHRONOUS) {	// iso in endpoint
					uint32_t size;
					ISO_Address = (uint8_t *) CALLBACK_HAL_GetISOBufferAddress(n, &size);
					DcdDataTransfer(2 * n + 1, ISO_Address, size);
				}
				else {
					if (stream_remain_packets > 0) {
						uint32_t cnt = dQueueHead[2 * n].TransferCount;
						Endpoint_Streaming((uint8_t *) stream_buffer_address,
										   stream_packet_size,
										   stream_remain_packets,
										   0);
						dQueueHead[2 * n].TransferCount = cnt;
					}
					else {
						stream_total_packets = 0;
					}
				}
				EVENT_USB_Device_TransferComplete(n, 1);
			}
		}
	}
}

void Endpoint_GetSetupPackage(uint8_t *pData)
{
	USB_Request_Header_t *ctrlrq = (USB_Request_Header_t *) pData;
	memcpy(pData, (void *) dQueueHead[0].SetupPackage, 8);
	/* Below fix is to prevent Endpoint_Read_Control_Stream_LE()
	 * from getting wrong data*/

	if (
		(ctrlrq->wLength != 0)
		) {
		dQueueHead[0].IsOutReceived = 0;
	}
}

void DcdIrqHandler(uint8_t HostID)
{
	uint32_t USBSTS_D;
        uint32_t t = USB_REG(USBPortNum)->USBINTR_D;

	USBSTS_D = USB_REG(USBPortNum)->USBSTS_D & t;	/* Device Interrupt Status */
	if (USBSTS_D == 0) {/* avoid to clear disabled interrupt source */
		return;
	}

	USB_REG(USBPortNum)->USBSTS_D = USBSTS_D;	/* Acknowledge Interrupt */

	/* Process Interrupt Sources */
	if (USBSTS_D & USBSTS_D_UsbInt) {
		if (USB_REG(USBPortNum)->ENDPTSETUPSTAT) {
			//			memcpy(SetupPackage, dQueueHead[0].SetupPackage, 8);
			/* Will be cleared by Endpoint_ClearSETUP */
		}

		if (USB_REG(USBPortNum)->ENDPTCOMPLETE) {
			TransferCompleteISR();
		}
	}

	if (USBSTS_D & USBSTS_D_NAK) {					/* NAK */
		uint32_t ENDPTNAK = USB_REG(USBPortNum)->ENDPTNAK;
                uint32_t en = USB_REG(USBPortNum)->ENDPTNAKEN;
                ENDPTNAK &= en;
		USB_REG(USBPortNum)->ENDPTNAK = ENDPTNAK;

		if (ENDPTNAK) {	/* handle NAK interrupts */
			uint8_t LogicalEP;
			for (LogicalEP = 0; LogicalEP < USED_PHYSICAL_ENDPOINTS / 2; LogicalEP++)
				if (ENDPTNAK & _BIT(LogicalEP)) {	/* Only OUT Endpoint is NAK enable */
					uint8_t PhyEP = 2 * LogicalEP;
					if ( !(USB_REG(USBPortNum)->ENDPTSTAT & _BIT(LogicalEP)) ) {/* Is In ready */
						/* Check read OUT flag */
						if (!dQueueHead[PhyEP].IsOutReceived) {

							if (PhyEP == 0) {
								usb_data_buffer_size = 0;
								USB_REG(USBPortNum)->ENDPTNAKEN &= ~(1 << 0);
								DcdDataTransfer(PhyEP, usb_data_buffer, 512);
							}
							else {
								if (stream_total_packets == 0) {
									usb_data_buffer_OUT_size = 0;
									/* Clear NAK */
									USB_REG(USBPortNum)->ENDPTNAKEN &= ~(1 << LogicalEP);
									DcdDataTransfer(PhyEP, usb_data_buffer_OUT, 512	/*512*/);
								}
							}
						}
					}
				}
		}
	}

	if (USBSTS_D & USBSTS_D_SofReceived) {					/* Start of Frame Interrupt */
		EVENT_USB_Device_StartOfFrame();
	}

	if (USBSTS_D & USBSTS_D_ResetReceived) {					/* Reset */
		HAL_Reset();
		USB_DeviceState = DEVICE_STATE_Default;
		Endpoint_ConfigureEndpoint(ENDPOINT_CONTROLEP,
								   EP_TYPE_CONTROL,
								   ENDPOINT_DIR_OUT,
								   USB_Device_ControlEndpointSize,
								   0);
		Endpoint_ConfigureEndpoint(ENDPOINT_CONTROLEP,
								   EP_TYPE_CONTROL,
								   ENDPOINT_DIR_IN,
								   USB_Device_ControlEndpointSize,
								   0);
	}

	if (USBSTS_D & USBSTS_D_SuspendInt) {					/* Suspend */

	}

	if (USBSTS_D & USBSTS_D_PortChangeDetect) {					/* Resume */

	}

	if (USBSTS_D & USBSTS_D_UsbErrorInt) {					/* Error Interrupt */
		// while(1){}
	}
}

uint32_t Dummy_EPGetISOAddress(uint32_t EPNum, uint32_t *last_packet_size)
{
	return (uint32_t) iso_buffer;
}

/*********************************************************************//**
 * @brief		Dummy USB device transfer complete event
 * @param[in]   logicalEP Logical endpoint number
 * @param[in]	xfer_in If this argument is 0 then xfer type is out, else in
 * @note		This event is required for running the stack with RTOS
 *      		and so it should never be removed!
 * @return	 	None
 **********************************************************************/
void Dummy_EVENT_USB_Device_TransferComplete(int logicalEP, int xfer_in)
{
	/**
	 * This is a dummy function
	 * If xfer_in zero then the endpoint it OUT
	 * else ep is IN.
	 **/
}
// #endif

#endif /*__LPC18XX__*/
